import click
import sys
from ziion_cli.dispatcher import *
from ziion_cli.__version__ import __version__
import platform
from ziion_cli.constants import (
    METAPACKAGES,
)

@click.group()
@click.version_option(version=__version__) 
def commands():
    pass

def solc() -> None:
    solc_imp()
    

def main(args=None):
    """The main routine."""
    if args is None:
        args = sys.argv[1:]
    commands()
    
@click.command()
def list_metapackages():
    for key in METAPACKAGES.keys() :
        click.echo(key)

@click.command()
def self_update():
    update_cli()
    

@click.command()
@click.argument("metapackage", type=click.Choice(METAPACKAGES.keys()), nargs=-1)
@click.option("--dryrun",is_flag=True, help="Flag to preview the packages to be updated.")
def update(metapackage, dryrun):
    if metapackage == ():
        click.echo("Error, at least on package from list_metapackages must be specified!")
        exit
    for package in metapackage:
        update_packages(METAPACKAGES.get(package), dryrun)

@click.command()
@click.argument("version", required=False)
@click.option("--install",is_flag=True, help="Flag to install the artifact if not installed yet.")
@click.option("--versions",is_flag=True, help="Flag to preview the installed solc versions.")
def solc_select(version, install, versions):
    if versions:
        solc_select_get_versions()
        pass
    elif version == None:
        click.echo("Missing version to be selected.")
    else:
        solc_select_imp(version, install)

commands.add_command(list_metapackages)
commands.add_command(self_update)
commands.add_command(update)
if platform.machine() == 'aarch64':
    commands.add_command(solc_select)

if __name__ == "__main__":
    main()


import subprocess

def self_update():
    print("updating ziion-cli...")
    print("\nCurrent version:")
    subprocess.run(["ziion", "--version"],
            stderr=subprocess.STDOUT)
    print("\n")
    subprocess.run(["sudo", "pip3.10", "install", "ziion", "-U"],
            stderr=subprocess.STDOUT)
    print("\nNew version:")
    subprocess.run(["ziion", "--version"],
            stderr=subprocess.STDOUT)


import ziion_cli.cargo
import ziion_cli.solc_select
import ziion_cli.utils
import ziion_cli.ziion
import subprocess
import sys
from ziion_cli.constants import (
    SOLC_SELECT_DIR,
    SOLC_ARTIFACTS_DIR
)

def update_packages(metapackage, dryrun):
    match metapackage:
        case "cargo":
            s3_packages_list = ziion_cli.utils.get_metadata_versions(metapackage)
            local_packages_list = ziion_cli.cargo.installed_versions()
            if dryrun:
                ziion_cli.cargo.list_packages_to_be_updated(s3_packages_list, local_packages_list)
            else:
                ziion_cli.cargo.update_necessary_packages(s3_packages_list, local_packages_list)
        case "solc":
            s3_packages_list = ziion_cli.utils.get_metadata_versions(metapackage)
            local_packages_list = ziion_cli.solc_select.installed_versions()
            missing_artifacts = []
            for i in s3_packages_list:
                if i not in local_packages_list:
                    missing_artifacts.append(i)
            if dryrun:
                print("These versions can be installed: ")
                for version in missing_artifacts:
                    print("- " + version)
            elif not dryrun and missing_artifacts != []:
                ziion_cli.solc_select.install_artifacts(missing_artifacts)
            else:
                print("Solc artifacts are up to date!")

def solc_select_imp(version, install='False' ):
    ziion_cli.solc_select.switch_global_version(version, install)

def solc_select_get_versions(): 
    try:
        with open(f"{SOLC_SELECT_DIR}/global-version", "r", encoding="utf-8") as f:
            current_version = f.read()
        for i in ziion_cli.solc_select.installed_versions():
            if current_version == i:
                print(i + " (current, set by " + str(SOLC_SELECT_DIR) + "/global-version)")
            else:
                print(i)
    except FileNotFoundError as e:
        print("No solc version selected for current usage. Use ziion solc-select [Version] first.")

def solc_imp():
    res = ziion_cli.solc_select.current_version()
    if res:
        (version, _) = res
        path = SOLC_ARTIFACTS_DIR.joinpath(f"solc-{version}")
        try:
            subprocess.run(
                [str(path)] + sys.argv[1:],
                check=True,
            )
        except subprocess.CalledProcessError as e:
            sys.exit(e.returncode)
    else:
        sys.exit(1)

def update_cli():
    ziion_cli.ziion.self_update()



import os
from ziion_cli.constants import (
    SOLC_SELECT_DIR,
    SOLC_ARTIFACTS_DIR,
)

if not os.path.exists(SOLC_SELECT_DIR):
    os.mkdir(path=SOLC_SELECT_DIR)

if not os.path.exists(SOLC_ARTIFACTS_DIR):
    os.mkdir(path=SOLC_ARTIFACTS_DIR)



from setuptools import setup
from ziion_cli.__version__ import __version__
import os
import platform

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

def define_entry_points():
    if platform.machine() == 'x86_64':
        entry_points = {
            "console_scripts": [
            "ziion = ziion_cli.__main__:main",
            ]
        }
    elif platform.machine() == 'aarch64': 
        entry_points = {
            "console_scripts": [
            "ziion = ziion_cli.__main__:main",
            "solc = ziion_cli.__main__:solc",
            ]
        }
    return entry_points

setup(
    name="ziion",
    author = "Halborn",
    description = ("ziion-cli provides an easy way to manage rust and solc packages for ARM and AMD."),
    version=__version__,
    install_requires=["packaging", "click"],
    packages=["ziion_cli"],
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    entry_points=define_entry_points()
)


from pathlib import Path
import platform

HOME_DIR = Path.home()

CARGO_DIR = HOME_DIR.joinpath(".cargo")
CARGO_ARTIFACTS_DIR = HOME_DIR.joinpath(".cargo/bin")
SOLC_SELECT_DIR = HOME_DIR.joinpath(".ziion-solc-select")
SOLC_ARTIFACTS_DIR = HOME_DIR.joinpath(".solcx")

S3_BUCKET_URL="https://ziion-binaries.s3.amazonaws.com/"
CARGO_AMD_FOLDER_S3="rust-amd-binaries/"
CARGO_ARM_FOLDER_S3="rust-amd-binaries/"
SOLC_ARM_FOLDER_S3="solc-arm-binaries/"

if platform.machine() == 'x86_64':
    METAPACKAGES = {
        "cargo": "cargo"
        }
elif platform.machine() == 'aarch64':
    METAPACKAGES = {
        "cargo": "cargo",
        "solc": "solc"
        }



from packaging import version
import subprocess
import platform
import ziion_cli.dispatcher
import ziion_cli.utils
from ziion_cli.constants import (
    CARGO_ARTIFACTS_DIR,
    CARGO_DIR
)

def installed_versions():
    output = subprocess.check_output(["cargo","install","--list"])
    return ziion_cli.utils.parse_str_to_dict(output.decode("utf-8"))


def list_packages_to_be_updated(s3_packages_list, local_packages_list):
    packages_to_update=[]
    print ("\n{:<30} {:<15} {:<15}".format('Package','Installed','Latest'))
    print ("-"*55)
    for package in s3_packages_list:
        if package not in local_packages_list:
            print ("{:<30} {:<15} {:<15}".format(package," No ", s3_packages_list[package]))
            packages_to_update.append(package)
        elif package in local_packages_list:
            print ("{:<30} {:<15} {:<15}".format(package,local_packages_list[package], s3_packages_list[package]))
        elif version.parse(s3_packages_list[package]) > version.parse(local_packages_list[package]):
            print ("{:<30} {:<15} {:<15}".format(package,local_packages_list[package], s3_packages_list[package]))
            packages_to_update.append(package)
    print("\n")
    return packages_to_update

def update_necessary_packages(s3_packages_list, local_packages_list):
    print("\nPackages to be updated:")
    packages = list_packages_to_be_updated(s3_packages_list, local_packages_list)
    if platform.machine() == 'x86_64':
        s3_folder = "rust-amd-binaries/"
    elif platform.machine() == 'aarch64':
        s3_folder = "rust-arm-binaries/"
    for package in packages:
        download = subprocess.run(["wget", "-O", CARGO_ARTIFACTS_DIR.joinpath(package), "https://ziion-binaries.s3.amazonaws.com/"+ s3_folder + package], 
            stdout=subprocess.DEVNULL,
            stderr=subprocess.STDOUT)
        subprocess.run(["chmod", "755", CARGO_ARTIFACTS_DIR.joinpath(package)], 
            stdout=subprocess.DEVNULL,
            stderr=subprocess.STDOUT)
        if download.returncode == 0:
            print(package + " updated succesfully.")
        else:
            print(package + " could not be updated correctly.")
    if packages: 
        subprocess.run(["wget", "-O", str(CARGO_DIR) + "/.crates.toml", "https://ziion-binaries.s3.amazonaws.com/"+ s3_folder + ".crates.toml"], 
                stdout=subprocess.DEVNULL,
                stderr=subprocess.STDOUT)
    print("\n")


import subprocess
import re
import json
import platform

def parse_str_to_dict(s) -> (dict):
    packages = {}
    lines = s.splitlines()
    for line in lines:
        if ":" in line:
            print(line)
            segments = line.split()
            version = segments[1].replace(':','')
        elif ":" not in line:
            packages[line.strip()] = version    
    return packages

def get_metadata_versions(metapackage="cargo") -> (dict):
    if metapackage == 'solc':
        s3_folder = "solc-arm-binaries/"
    elif metapackage == 'cargo':
        if platform.machine() == 'x86_64':
            s3_folder = "rust-amd-binaries/"
        elif platform.machine() == 'aarch64':
            s3_folder = "rust-arm-binaries/"
    subprocess.run(["wget", "-O", "metadata.json", "https://ziion-binaries.s3.amazonaws.com/"+ s3_folder + "metadata.json"], 
            stdout=subprocess.DEVNULL,
            stderr=subprocess.STDOUT)
    f = open('metadata.json')
    data = json.load(f)
    return data["releases"]




import os
from pathlib import Path
import urllib.request
import ziion_cli.utils
import argparse
from ziion_cli.constants import (
    SOLC_SELECT_DIR,
    SOLC_ARTIFACTS_DIR,
    S3_BUCKET_URL,
    SOLC_ARM_FOLDER_S3
)

def switch_global_version(version: str, always_install: bool) -> None:
    if version in installed_versions():
        with open(f"{SOLC_SELECT_DIR}/global-version", "w", encoding="utf-8") as f:
            f.write(version)
        print("Switched global version to", version)
    elif version in ziion_cli.utils.get_metadata_versions("solc"):
        if always_install:
            install_artifacts([version])
            switch_global_version(version, always_install)
        else:
            print(f"ziion-cli solc-select error: '{version}' must be installed prior to use.")
    else:
        print(f"ziion-cli solc-select error: Unknown version '{version}'")

def installed_versions() -> list[str]:
    try:
        return [
            f.replace("solc-", "") for f in sorted(os.listdir(SOLC_ARTIFACTS_DIR)) if f.startswith("solc-")    
        ]
    except FileNotFoundError as e:
        return []

def install_artifacts(versions: list[str]) -> bool:
    #releases = utils.get_metadata_versions("solc")

    for version in versions:
        if "all" not in versions:
            if versions and version not in versions:
                continue

        url = S3_BUCKET_URL + SOLC_ARM_FOLDER_S3 + "solc-v" + version
        print(f"Installing '{version}'...")
        try:      
            urllib.request.urlretrieve(url, SOLC_ARTIFACTS_DIR.joinpath(f"solc-{version}"))
        except urllib.error.HTTPError as e:     
            print(e.reason)
        
        with open(f"{SOLC_SELECT_DIR}/global-version", "w+", encoding="utf-8") as f:
            f.write(version)

        #verify_checksum(version)

        Path.chmod(SOLC_ARTIFACTS_DIR.joinpath(f"solc-{version}"), 0o775)
        print(f"Version '{version}' installed and configured as default.\n")

    return True


def current_version():
    version = os.environ.get("SOLC_VERSION")
    source = "SOLC_VERSION"
    if version:
        if version not in installed_versions():
            raise argparse.ArgumentTypeError(
                f"Version '{version}' not installed (set by {source}). Run `solc-select install {version}`."
            )
    else:
        source = SOLC_SELECT_DIR.joinpath("global-version")
        if Path.is_file(source):
            with open(source, encoding="utf-8") as f:
                version = f.read()
        else:
            raise argparse.ArgumentTypeError(
                "No solc version set. Run `solc-select use VERSION` or set SOLC_VERSION environment variable."
            )
    return version, source

